"""
The methods to run the parts of the form compiler

Should also contain the entrypoint methods.
"""

def read_ufl(uflfile):
    from ufl.algorithms.formfiles import read_ufl_file, interpret_ufl_namespace
    from ufl.algorithms import compute_form_data

    uflcode = read_ufl_file(uflfile)
    namespace = {}
    uflcode = "from dune.perftool.runtime_ufl import *\n" + uflcode
    exec uflcode in namespace
    data = interpret_ufl_namespace(namespace)
    formdata = compute_form_data(data.forms[0])
    return formdata


def generate_driver(formdata, filename):
    # The driver module uses a global variable for ease of use
    from dune.perftool.pdelab.driver import set_formdata
    set_formdata(formdata)

    # The vtkoutput is the generating method that triggers all others.
    # Alternatively, one could use the `solve` method.
    from dune.perftool.pdelab.driver import vtkoutput
    vtkoutput()

    from dune.perftool.generation import retrieve_cache_items

    # Get all preambles for the driver and sort them.
    driver_content = [i[1] for i in sorted(retrieve_cache_items(item_name="driver_preamble"), key=lambda x : x[0])]

    # And flatten out those, that contained nested lists
    def flatjoin(l):
        if isinstance(l, str):
            return "\n" + l
        return "".join(flatjoin(i) for i in l)

    from cgen import LiteralBlock
    driver = LiteralBlock(flatjoin(driver_content))

    # Write the file.
    f = open(filename, 'w')
    f.write("\n".join(retrieve_cache_items(item_name="driver_include")))
    f.write("\n\nvoid driver(int argc, char** argv)\n")
    f.write("\n".join(driver.generate()))

    # Reset the caching data structure
    from dune.perftool.generation import delete_cache
    delete_cache()


def compile_form():
    from dune.perftool.options import get_option
    ufldata = read_ufl(get_option("uflfile"))

    if get_option("driver_file"):
        generate_driver(ufldata, get_option("driver_file"))

    if get_option("operator_file"):
        from dune.perftool.pdelab.localoperator import generate_localoperator
        generate_localoperator(ufldata, get_option("operator_file"))
